/*
there's no LCD detection, so for debug mode without the LCD, pull D0 to GND before power on.  
  LED will come on until you remove the jumper, at which point Serial will be initialized
  and pins returned to defaults

*/
//uncomment this for dev mode
//#define DEVMODE 1

#include <myUTFT.h> //http://www.henningkarlsen.com/electronics
#include <DHT22.h> //https://github.com/ringerc/Arduino-DHT22
#include <Wire.h>

#define I2C_SLAVE 0x29
#define DHT22_PIN 8

// Declare which fonts we will be using
extern uint8_t BigFont[];

myUTFT myGLCD(ITDB28,9,10,11,12);   // Remember to change the model parameter to suit your display module!

// Setup a DHT22 instance
DHT22 myDHT22(DHT22_PIN);
long lDHTlastread = 0;        // will store last time sensor was read
long DHTinterval = 3000;           // interval at which to read dht (milliseconds)

//i2c buffers
String i2cbuffer;
String i2cOutput;
boolean haveData = false;
int i2cSendIndex = 0;
long lastI2C = 0;

void setup(){
#if defined(DEVMODE)
    Serial.begin(115200);
    Serial.print("Devmode ON");
#else
    pinMode(13, INPUT); //return to its default state
  // Setup the LCD
    myGLCD.InitLCD();
    myGLCD.setFont(BigFont);
  
    myGLCD.clrScr();
  
    myGLCD.fillScr(colorcode("PUR"));
    drawText("Initializing...", CENTER, 111, colorcode("GRE"), colorcode("BLA"));
#endif
  i2cbuffer = "";
  i2cOutput = "";
  Wire.begin(I2C_SLAVE);
  Wire.onRequest(sendI2Cdata); 
  Wire.onReceive(recvI2Cdata); 
  lastI2C = millis();
#if defined(DEVMODE)
  Serial.println("started");
#endif
}

void loop() {
// Clear the screen and draw the frame

  getDHT();
/*  if(getDHT()) {
    if(millis() - lDHTlastread < 100) {
#if defined(DEVMODE)
      Serial.print("Temp F ");
      Serial.println((myDHT22.getTemperatureC() * 9.0)/ 5.0 + 32.0);
      
      Serial.print("Humidity %");
      Serial.println(myDHT22.getHumidity());
#else
      myGLCD.print("   Temp F ", 0, 20);
      myGLCD.printNumF((myDHT22.getTemperatureC() * 9.0)/ 5.0 + 32.0, 1, 160, 20);
      
      myGLCD.print("Humidity%", 0, 40);
      myGLCD.printNumF(myDHT22.getHumidity(), 1, 160, 40);
#endif
      delay(50);
    }
  } else {
#if defined(DEVMODE)
    Serial.println("DHT22 not found");
#else
    //nada
#endif
  }
*/
  if (haveData) {
#if defined(DEVMODE)
    Serial.print("i2cbuffer: ");
    Serial.println(i2cbuffer);
    Serial.print("len: ");
    Serial.println(i2cbuffer.length());
#endif
    processBuffer();
    i2cbuffer = "";
    haveData = false;
  }
  
  if ((i2cbuffer != "" || i2cSendIndex > 0) && millis() > lastI2C + 1000 ) {//coms idled?
    i2cbuffer = "";
    i2cSendIndex = 0;
    haveData = false;
    lastI2C = millis();
  }
  if (millis() > lastI2C + 10000 ) {//coms dead?
    myGLCD.fillScr(colorcode("PUR"));
    drawText("Check Host", CENTER, 111, colorcode("GRE"), colorcode("BLA"));
    lastI2C = millis() - 5000; //causes screen to redraw every 5 seconds instead of continious
  }
}

void processBuffer() {
  char temp[50];
  char *last;
  int x1; int y1; int x2; int y2; word bordercolor; word fillcolor; boolean rounded;
  
  if(i2cbuffer == "TEMP") {
    dtostrf(((myDHT22.getTemperatureC() * 9.0)/ 5.0 + 32.0),1,2,temp);
    i2cOutput = String(temp);
  } else if(i2cbuffer == "HUMID") {
    dtostrf(myDHT22.getHumidity(),1,2,temp);
    i2cOutput = String(temp);
  } else if(i2cbuffer.substring(0,3)  == "CLR"){
    i2cbuffer.substring(4).toCharArray(temp, 25); 
    bordercolor = colorcode(strtok_r(temp, "~", &last));  //color
    drawClr(bordercolor);
  } else if(i2cbuffer.substring(0,3)  == "DOT"){
    i2cbuffer.substring(4).toCharArray(temp, 25); 
    x1 = atoi(strtok_r(temp, "~", &last)); //x1
    y1 = atoi(strtok_r(NULL, "~", &last));  //y1
    bordercolor = colorcode(strtok_r(NULL, "~", &last));  //color
    drawPoint(x1, y1, bordercolor);
  } else if(i2cbuffer.substring(0,4)  == "LINE"){
    i2cbuffer.substring(5).toCharArray(temp, 25); 
    x1 = atoi(strtok_r(temp, "~", &last)); //x1
    y1 = atoi(strtok_r(NULL, "~", &last));  //y1
    x2 = atoi(strtok_r(NULL, "~", &last));  //x2
    y2 = atoi(strtok_r(NULL, "~", &last));  //y2
    bordercolor = colorcode(strtok_r(NULL, "~", &last));  //color
    drawLine(x1, y1, x2, y2, bordercolor);
  } else if(i2cbuffer.substring(0,4)  == "RECT"){
    i2cbuffer.substring(5).toCharArray(temp, 25); 
    x1 = atoi(strtok_r(temp, "~", &last)); //x1
    y1 = atoi(strtok_r(NULL, "~", &last));  //y1
    x2 = atoi(strtok_r(NULL, "~", &last));  //x2
    y2 = atoi(strtok_r(NULL, "~", &last));  //y2
    bordercolor = colorcode(strtok_r(NULL, "~", &last));  //color
    fillcolor = colorcode(strtok_r(NULL, "~", &last));  //color
    rounded = atoi(strtok_r(NULL, "~", &last));  //rounded
    drawRect(x1, y1, x2, y2, bordercolor, fillcolor, rounded);
  } else if(i2cbuffer.substring(0,4)  == "CIRC"){
    i2cbuffer.substring(5).toCharArray(temp, 25); 
    x1 = atoi(strtok_r(temp, "~", &last)); //x1
    y1 = atoi(strtok_r(NULL, "~", &last));  //y1
    x2 = atoi(strtok_r(NULL, "~", &last));  //rad
    bordercolor = colorcode(strtok_r(NULL, "~", &last));  //color
    fillcolor = colorcode(strtok_r(NULL, "~", &last));  //color
    drawCircle(x1, y1, x2, bordercolor, fillcolor);
  } else if(i2cbuffer.substring(0,4)  == "TEXT"){
    i2cbuffer.substring(5).toCharArray(temp, 50); 
    String text;
    text.concat(strtok_r(temp, "~", &last)); //text
    x1 = atoi(strtok_r(NULL, "~", &last)); //x1
    y1 = atoi(strtok_r(NULL, "~", &last));  //y1
    bordercolor = colorcode(strtok_r(NULL, "~", &last));  //color
    fillcolor = colorcode(strtok_r(NULL, "~", &last));  //color
    drawText(text, x1, y1, bordercolor, fillcolor);
  } else {
    i2cOutput = "";
  }
}

int getDHT() {
  DHT22_ERROR_t errorCode;

  unsigned long currentMillis = millis();
 
  if(currentMillis - lDHTlastread > DHTinterval) {
    lDHTlastread = currentMillis;
    errorCode = myDHT22.readData();
    switch(errorCode)
    {
      case DHT_ERROR_NONE:
        return 1;
        break;
      case DHT_ERROR_CHECKSUM:
#if defined(DEVMODE)
        Serial.print("HUMIDERR~");
        Serial.print("check sum error ");
        Serial.print(myDHT22.getTemperatureC());
        Serial.print("C ");
        Serial.print(myDHT22.getHumidity());
        Serial.println("%");
#endif
        break;
      case DHT_BUS_HUNG:
#if defined(DEVMODE)
        Serial.print("HUMIDERR~");
        Serial.println("BUS Hung ");
#endif
        break;
      case DHT_ERROR_NOT_PRESENT:
#if defined(DEVMODE)
        Serial.print("HUMIDERR~");
        Serial.println("Not Present ");
#endif
        break;
      case DHT_ERROR_ACK_TOO_LONG:
#if defined(DEVMODE)
        Serial.print("HUMIDERR~");
        Serial.println("ACK time out ");
#endif
        break;
      case DHT_ERROR_SYNC_TIMEOUT:
#if defined(DEVMODE)
        Serial.print("HUMIDERR~");
        Serial.println("Sync Timeout ");
#endif
        break;
      case DHT_ERROR_DATA_TIMEOUT:
#if defined(DEVMODE)
        Serial.print("HUMIDERR~");
        Serial.println("Data Timeout ");
#endif
        break;
      case DHT_ERROR_TOOQUICK:
#if defined(DEVMODE)
        Serial.print("HUMIDERR~");
        Serial.println("Polled to quick ");
#endif
        break;
    }
    return 0;
  }
  else
  {
    return 1; //should keep the cache
  }
}

// callback for sending data
void sendI2Cdata() { 
  lastI2C = millis();
  if (i2cSendIndex >= i2cOutput.length()) {
      Wire.write(0); //send a null terminator
      i2cSendIndex = 0;
      i2cOutput = "";
  } else {
    Wire.write(i2cOutput[i2cSendIndex]);
    ++i2cSendIndex;
  }
}

void recvI2Cdata(int datalen) {
  lastI2C = millis();
  char b;
  b = Wire.read();
  if (b == 0) {
    haveData = true;
  } else {
    i2cbuffer += b;
  }
}

void drawClr(word color) {
    myGLCD.clrScr();
    myGLCD.fillScr(color);
}

void drawPoint(int x, int y, word color) {
  myGLCD.setColor(color);
  myGLCD.drawPixel(x,y);
}

void drawLine(int x1, int y1, int x2, int y2, word color) {
  myGLCD.setColor(color);
  myGLCD.drawLine(x1, y1, x2, y2);
}

void drawRect(int x1, int y1, int x2, int y2, word bordercolor, word fillcolor, boolean rounded) {
  if (fillcolor == VGA_TRANSPARENT) { //make a border only
    myGLCD.setColor(bordercolor);
    if(rounded) {
      myGLCD.drawRoundRect(x1, y1, x2, y2);
    } else {
      myGLCD.drawRect(x1, y1, x2, y2);
    }
  } else { //filled
    myGLCD.setColor(fillcolor);
    if(rounded) {
      myGLCD.fillRoundRect(x1, y1, x2, y2);
    } else {
      myGLCD.fillRect(x1, y1, x2, y2);
    }
    if(fillcolor != bordercolor) {  //and a different collor border
      myGLCD.setColor(bordercolor);
      if(rounded) {
        myGLCD.drawRoundRect(x1, y1, x2, y2);
      } else {
        myGLCD.drawRect(x1, y1, x2, y2);
      }
    }
  }
}

void drawCircle(int x, int y, int rad, word bordercolor, word fillcolor) {
  if (fillcolor == VGA_TRANSPARENT) { //make a border only
    myGLCD.setColor(bordercolor);
    myGLCD.drawCircle(x, y, rad);
  } else { //filled
    myGLCD.setColor(fillcolor);
    myGLCD.fillCircle(x, y, rad);
    if(fillcolor != bordercolor) {  //and a different collor border
      myGLCD.setColor(bordercolor);
      myGLCD.drawCircle(x, y, rad);
    }
  }
}

void drawText(String text, int x, int y, word color, word backcolor) {
  if(x == -1) x = CENTER;
  myGLCD.setColor(color);
  myGLCD.setBackColor(backcolor);
  myGLCD.print(text, x, y);
}

word colorcode(char *col) {
  if(strcmp(col, "BLA") == 0) {
    return VGA_BLACK;
  } else if(strcmp(col, "MAR") == 0) {
    return VGA_MAROON;
  } else if(strcmp(col, "GRE") == 0) {
    return VGA_GREEN;
  } else if(strcmp(col, "NAV") == 0) {
    return VGA_NAVY;
  } else if(strcmp(col, "SIL") == 0) {
    return VGA_SILVER;
  } else if(strcmp(col, "GRA") == 0) {
    return VGA_GRAY;
  } else if(strcmp(col, "RED") == 0) {
    return VGA_RED;
  } else if(strcmp(col, "PUR") == 0) {
    return VGA_PURPLE;
  } else if(strcmp(col, "LIM") == 0) {
    return VGA_LIME;
  } else if(strcmp(col, "OLI") == 0) {
    return VGA_OLIVE;
  } else if(strcmp(col, "BLU") == 0) {
    return VGA_BLUE;
  } else if(strcmp(col, "TEA") == 0) {
    return VGA_TEAL;
  } else if(strcmp(col, "WHI") == 0) {
    return VGA_WHITE;
  } else if(strcmp(col, "FUC") == 0) {
    return VGA_FUCHSIA;
  } else if(strcmp(col, "YEL") == 0) {
    return VGA_YELLOW;
  } else if(strcmp(col, "AQU") == 0) {
    return VGA_AQUA;
  } else {
    drawText("TRANS!",0,220,VGA_AQUA,VGA_BLACK);
   return VGA_TRANSPARENT; //  - use this for fillcolor to denote no fill
  }     
}
